import sys,time
from os import environ
from os import getcwd
import string
sys.path.append(environ["PYTHON_MODULE_PATH"])
sys.path.append(environ["SWIG_LIB_INSTALL_DIR"])

global Lx; global Ly; global cd; global T; global nOrder
global LamV; global LamS; global tV; global tS

#PARAMETERS:
cd=7            #typical cell diameter
Lx=5*cd         #Lattice size - x
Ly=5*cd         #Lattice size - y
T=10           #Temperature
nOrder=4        #Distance of interaction
#
#VOLUME/SURFACE PARAMETERS:
LamV=10        #Lambda Volume
tV=cd*cd       #Target Volume

def configureSimulation(sim):
   import CompuCellSetup
   from XMLUtils import ElementCC3D
   cc3d=ElementCC3D("CompuCell3D")
   potts=cc3d.ElementCC3D("Potts")
   potts.ElementCC3D("Dimensions",{"x":Lx,"y":Ly,"z":1})
   potts.ElementCC3D("Steps",{},10000)
   potts.ElementCC3D("Temperature",{},int(T))
   potts.ElementCC3D("NeighborOrder",{},2)
   potts.ElementCC3D("Boundary_x",{},"Periodic")
   potts.ElementCC3D("Boundary_y",{},"Periodic")
   
   cellType=cc3d.ElementCC3D("Plugin",{"Name":"CellType"})
   cellType.ElementCC3D("CellType", {"TypeName":"Medium","TypeId":"0"})
   cellType.ElementCC3D("CellType", {"TypeName":"TypeA", "TypeId":"1"})

   #CONTACT
   contact=cc3d.ElementCC3D("Plugin",{"Name":"Contact"})
   contact.ElementCC3D("Energy", {"Type1":"Medium", "Type2": "Medium"},0)
   contact.ElementCC3D("Energy", {"Type1":"Medium", "Type2": "TypeA"},10)
   contact.ElementCC3D("Energy", {"Type1":"TypeA",  "Type2": "TypeA"},10)
   #-neighbor order
   contact.ElementCC3D("NeighborOrder",{},nOrder)
   
   #OTHER PLUGINS
   vlfpd = cc3d.ElementCC3D("Plugin",{"Name":"VolumeLocalFlex"})
   ntpd = cc3d.ElementCC3D("Plugin",{"Name":"NeighborTracker"})
   
   #CELLS INITIAL CONFIGURATION:
   uipd = cc3d.ElementCC3D("Steppable",{"Type":"UniformInitializer"})
   region = uipd.ElementCC3D("Region")
   region.ElementCC3D("BoxMin",{"x":Lx/2-1,  "y":Ly/2-1,  "z":0})
   region.ElementCC3D("BoxMax",{"x":Lx/2+1,  "y":Ly/2+1,  "z":1})
   region.ElementCC3D("Types",{}, "TypeA")
   region.ElementCC3D("Width", {}, 4)
   
   #BOX WATCHER
   bwpd=cc3d.ElementCC3D("Steppable",{"Type":"BoxWatcher"})
   bwpd.ElementCC3D("XMargin",  {},  2)
   bwpd.ElementCC3D("YMargin",  {},  2)
   bwpd.ElementCC3D("ZMargin",  {},  1)
      
   CompuCellSetup.setSimulationXMLDescription(cc3d)


import CompuCellSetup
sim,simthread = CompuCellSetup.getCoreSimulationObjects()
configureSimulation(sim)

import CompuCell
CompuCellSetup.initializeSimulationObjects(sim,simthread)
pyAttributeAdder,dictAdder=CompuCellSetup.attachDictionaryToCells(sim)

#Add Python steppables here
steppableRegistry=CompuCellSetup.getSteppableRegistry()

from CellCycle_Step import InitCond
initCond=InitCond(_simulator=sim,_frequency=1,_LamV=LamV,_tV=tV)
steppableRegistry.registerSteppable(initCond)

from CellCycle_Step import Cycle
cycle=Cycle(_simulator=sim,_frequency=1,_tV=tV)
steppableRegistry.registerSteppable(cycle)

from CellCycle_Step import MitosisSteppable
mitosisSteppable=MitosisSteppable(_simulator=sim,_frequency=1,_tV=tV)
steppableRegistry.registerSteppable(mitosisSteppable)

#Create extra player fields here or add attributes
dim=sim.getPotts().getCellFieldG().getDim()
CycField=simthread.createScalarFieldCellLevelPy("Cyc")

from CellCycle_Step import ExtraFields
extraFields=ExtraFields(_simulator=sim,_frequency=1)
extraFields.setScalarFields(CycField)
steppableRegistry.registerSteppable(extraFields)

CompuCellSetup.mainLoop(sim,simthread,steppableRegistry)
##sys.exit()
