import os
import sys
import CompuCell
from PySteppables import *
from PlayerPython import *
from PySteppablesExamples import MitosisSteppableBase
from copy import deepcopy
from math import *
import random


class InitCond(SteppableBasePy):
  def __init__(self,_simulator,_frequency,_LamV,_tV):
    SteppableBasePy.__init__(self,_simulator,_frequency)
    self.LamV=_LamV; self.tV=_tV
    
  def start(self):
    for cell in self.cellList:
      if (cell):
        cell.targetVolume=cell.volume
        cell.lambdaVolume=self.LamV
        
  def step(self,mcs):
    for cell in self.cellList:
      if (cell):
        if (cell.volume<self.tV):
          cell.targetVolume+=0.1 #0.01*cell.volume(1-cell.volume/416)

          
import bionetAPI
class Cycle(SteppableBasePy):
  def __init__(self,_simulator,_frequency,_tV):
    SteppableBasePy.__init__(self,_simulator,_frequency)
    bionetAPI.initializeBionetworkManager(self.simulator)
    self.tV=_tV
    
  def start(self):
    #Loading model
    ModelName = "CellCycle"
    ModelKey  = "CC"
    ModelPath = os.getcwd() + "/DemosBionetSolver/CellCycle/BIOMD0000000003.xml"
    IntegrationStep = 0.05
    bionetAPI.loadSBMLModel(ModelName, ModelPath, ModelKey, IntegrationStep)
    
    #Initial conditions
    bionetAPI.addSBMLModelToTemplateLibrary(ModelName,"TypeA")
    bionetAPI.initializeBionetworks()
    for cell in self.cellList:
      cellDict=CompuCell.getPyAttrib(cell)
      cellDict["M"]=bionetAPI.getBionetworkValue("CC_M",cell.id)
      cellDict["C"]=bionetAPI.getBionetworkValue("CC_C",cell.id)
      
  def step(self,mcs):
    for cell in self.cellList:
      #Storing CycB and Cdh1 levels of each cell
      cellDict=CompuCell.getPyAttrib(cell)
      cellDict["M0"]=cellDict["M"]
      cellDict["M"]=bionetAPI.getBionetworkValue("CC_M",cell.id)
      cellDict["C"]=bionetAPI.getBionetworkValue("CC_C",cell.id)          
    #Iterating the ODEs
    bionetAPI.timestepBionetworks()

      
class MitosisSteppable(MitosisSteppableBase):
  def __init__(self,_simulator,_frequency,_tV):
    MitosisSteppableBase.__init__(self,_simulator,_frequency)
    self.tV=_tV;

  def step(self,mcs):
    cells2divide=[]
    for cell in self.cellList:
      cellDict=CompuCell.getPyAttrib(cell)
      if (cellDict["M"]>=0.7 and cellDict["M0"]<0.7):
        cells2divide.append(cell)
    for cell in cells2divide:
      self.divideCellRandomOrientation(cell)

  def updateAttributes(self):  
    parentCell=self.mitosisSteppable.parentCell
    childCell=self.mitosisSteppable.childCell
        
    #transfering attributes
    childCell.type=parentCell.type
    bionetAPI.copyBionetworkFromParent(parentCell,childCell)
    #volume
    parentCell.targetVolume=parentCell.volume
    childCell.targetVolume=childCell.volume
    childCell.lambdaVolume=parentCell.lambdaVolume
    #dictionaries
    parentCellDict=CompuCell.getPyAttrib(parentCell)
    childCellDict=CompuCell.getPyAttrib(childCell)
    for key,item in parentCellDict.items():
      if (key!="Bionetwork"): 
        childCellDict[key] = deepcopy(parentCellDict[key])

    
class ExtraFields(SteppableBasePy):
  def __init__(self,_simulator,_frequency=1):
    SteppableBasePy.__init__(self,_simulator,_frequency)

  def setScalarFields(self,_field1):
      self.scalarField1=_field1

  def step(self,mcs):
    clearScalarValueCellLevel(self.scalarField1)
    for cell in self.cellList:
      if (cell):
        cellDict=CompuCell.getPyAttrib(cell)
        fillScalarValueCellLevel(self.scalarField1,cell,cellDict["C"])
