import os
import sys
import CompuCell
from PySteppables import *
from PlayerPython import *
from PySteppablesExamples import MitosisSteppableBase
from copy import deepcopy
from math import *
import random


class InitCond(SteppableBasePy):
  def __init__(self,_simulator,_frequency,_LamV,_tV):
    SteppableBasePy.__init__(self,_simulator,_frequency)
    self.LamV=_LamV; self.tV=_tV
    
  def start(self):
    for cell in self.cellList:
      if (cell):
        cell.targetVolume=cell.volume
        cell.lambdaVolume=self.LamV
        
  def step(self,mcs):
    for cell in self.cellList:
      if (cell):
        if (cell.volume<self.tV):
          cell.targetVolume+=0.1

          
import bionetAPI
class CycleDeltaNotch(SteppableBasePy):
  def __init__(self,_simulator,_frequency,_tV):
    SteppableBasePy.__init__(self,_simulator,_frequency)
    bionetAPI.initializeBionetworkManager(self.simulator)
    self.tV=_tV
    
  def start(self):
    #Loading model 1
    ModelName = "CellCycle"
    ModelKey = "CC"
    ModelPath = os.getcwd() + "\DemosBionetSolver\CellCycleDeltaNotch\CellCycle_Tyson.sbml"
    IntegrationStep = 0.1
    bionetAPI.loadSBMLModel( ModelName, ModelPath, ModelKey, IntegrationStep ) 
    bionetAPI.addSBMLModelToTemplateLibrary(ModelName,"TypeA")
    
    #Loading model 2
    ModelName = "DeltaNotch"
    ModelKey = "DN"
    ModelPath = os.getcwd() + "\DemosBionetSolver\CellCycleDeltaNotch\DN_Collier.sbml"
    IntegrationStep = 0.1
    bionetAPI.loadSBMLModel( ModelName, ModelPath, ModelKey, IntegrationStep )    
    bionetAPI.addSBMLModelToTemplateLibrary(ModelName,"TypeA")
    
    #Initial conditions
    bionetAPI.initializeBionetworks()
    for cell in self.cellList:
      if cell:
        bionetAPI.setBionetworkValue("CC_m",cell.volume/self.tV,cell.id)
        cellDict=CompuCell.getPyAttrib(cell)
        cellDict["CycB"]=bionetAPI.getBionetworkValue("CC_CycB",cell.id)
        cellDict["Cdh1"]=bionetAPI.getBionetworkValue("CC_Cdh1",cell.id)
        D=random.uniform(0.9,1.0); cellDict["D"]=D
        N=random.uniform(0.9,1.0); cellDict["N"]=N
        bionetAPI.setBionetworkValue("DN_D",D,cell.id)
        bionetAPI.setBionetworkValue("DN_N",N,cell.id)
      
  def step(self,mcs):
    for cell in self.cellList:
      if (cell):
        #Setting volume parameter for each cell
        bionetAPI.setBionetworkValue("CC_m",cell.volume/self.tV,cell.id)
        #Storing CycB and Cdh1 levels of each cell
        cellDict=CompuCell.getPyAttrib(cell)
        cellDict["CycB0"]=cellDict["CycB"]
        cellDict["CycB"]=bionetAPI.getBionetworkValue("CC_CycB",cell.id)
        cellDict["Cdh1"]=bionetAPI.getBionetworkValue("CC_Cdh1",cell.id)
        #Averaging Detla
        D=0.0; nn=0
        cellNeighborList=self.getCellNeighbors(cell)
        for neighbor in cellNeighborList:
          if (neighbor.neighborAddress):
            nn+=1
            D+=bionetAPI.getBionetworkValue("DN_D",neighbor.neighborAddress.id)
        if (nn>0):
          D=D/nn
        #Setting average Detla for each cell
        bionetAPI.setBionetworkValue("DN_Davg",D,cell.id)
        #Storing Delta and Notch levels of each cell
        cellDict=CompuCell.getPyAttrib(cell)
        cellDict["D"]=D
        cellDict["N"]=bionetAPI.getBionetworkValue("DN_N",cell.id)        
    #Iterating the ODEs
    bionetAPI.timestepBionetworks()

      
class MitosisSteppable(MitosisSteppableBase):
  def __init__(self,_simulator,_frequency,_tV):
    MitosisSteppableBase.__init__(self,_simulator,_frequency)
    self.tV=_tV;

  def step(self,mcs):
    cells2divide=[]
    for cell in self.cellList:
      if cell:
        cellDict=CompuCell.getPyAttrib(cell)
        if (cellDict["CycB"]<=0.1 and cellDict["CycB0"]>0.1):
          cells2divide.append(cell)
    for cell in cells2divide:
      self.divideCellRandomOrientation(cell)

  def updateAttributes(self):  
    parentCell=self.mitosisSteppable.parentCell
    childCell=self.mitosisSteppable.childCell
     
    #transfering attributes
    childCell.type=parentCell.type
    bionetAPI.copyBionetworkFromParent(parentCell,childCell)
    
    parentCell.targetVolume=parentCell.volume
    childCell.targetVolume=childCell.volume
    childCell.lambdaVolume=parentCell.lambdaVolume
    
    parentCellDict=CompuCell.getPyAttrib(parentCell)
    childCellDict=CompuCell.getPyAttrib(childCell)
    for key,item in parentCellDict.items():
      print key
      if (key!="Bionetwork"): 
        childCellDict[key] = deepcopy(parentCellDict[key])

    
class ExtraFields(SteppableBasePy):
  def __init__(self,_simulator,_frequency=1):
    SteppableBasePy.__init__(self,_simulator,_frequency)

  def setScalarFields(self,_field1,_field2,_field3,_field4):
      self.scalarField1=_field1
      self.scalarField2=_field2
      self.scalarField3=_field3
      self.scalarField4=_field4

  def step(self,mcs):
    clearScalarValueCellLevel(self.scalarField1)
    clearScalarValueCellLevel(self.scalarField2)
    clearScalarValueCellLevel(self.scalarField3)
    clearScalarValueCellLevel(self.scalarField4)
    for cell in self.cellList:
      if (cell):
        cellDict=CompuCell.getPyAttrib(cell)
        fillScalarValueCellLevel(self.scalarField1,cell,cellDict["CycB"])
        fillScalarValueCellLevel(self.scalarField2,cell,cellDict["Cdh1"])
        fillScalarValueCellLevel(self.scalarField3,cell,cellDict["D"])
        fillScalarValueCellLevel(self.scalarField4,cell,cellDict["N"])