import os
import sys
import CompuCell
from PySteppables import *
from PlayerPython import *
from PySteppablesExamples import MitosisSteppableBase
from copy import deepcopy
from math import *
import random


class InitCond(SteppableBasePy):
  def __init__(self,_simulator,_frequency,_LamV,_tV):
    SteppableBasePy.__init__(self,_simulator,_frequency)
    self.LamV=_LamV; self.tV=_tV
    
  def start(self):
    for cell in self.cellList:
      if (cell):
        cell.targetVolume=cell.volume
        cell.lambdaVolume=self.LamV
        
  def step(self,mcs):
    for cell in self.cellList:
      if (cell):
        if (cell.volume<self.tV):
          cell.targetVolume+=0.1

          
import bionetAPI
class Cycle(SteppableBasePy):
  def __init__(self,_simulator,_frequency,_tV):
    SteppableBasePy.__init__(self,_simulator,_frequency)
    bionetAPI.initializeBionetworkManager(self.simulator)
    self.tV=_tV
    
  def start(self):
    #Loading model
    ModelName = "CellCycle"
    ModelKey = "CC"
    ModelPath =  os.getcwd() + "/DemosBionetSolver/CellCycleTyson/CellCycle_Tyson.sbml"
    IntegrationStep = 0.1
    bionetAPI.loadSBMLModel(ModelName, ModelPath, ModelKey, IntegrationStep) 
    
    bionetAPI.addSBMLModelToTemplateLibrary(ModelName,"TypeA")
    bionetAPI.initializeBionetworks()
    
    #Initial conditions
    for cell in self.cellList:
      if cell:
        bionetAPI.setBionetworkValue("CC_m",cell.volume/self.tV,cell.id)
        cellDict=CompuCell.getPyAttrib(cell)
        cellDict["CycB"]=bionetAPI.getBionetworkValue("CC_CycB",cell.id)
        cellDict["Cdh1"]=bionetAPI.getBionetworkValue("CC_Cdh1",cell.id)
      
  def step(self,mcs):
    for cell in self.cellList:
      if (cell):
        #Setting volume parameter for each cell
        bionetAPI.setBionetworkValue("CC_m",cell.volume/self.tV,cell.id)
        #Storing CycB and Cdh1 levels of each cell
        cellDict=CompuCell.getPyAttrib(cell)
        cellDict["CycB0"]=cellDict["CycB"]
        cellDict["CycB"]=bionetAPI.getBionetworkValue("CC_CycB",cell.id)
        cellDict["Cdh1"]=bionetAPI.getBionetworkValue("CC_Cdh1",cell.id)        
    #Iterating the ODEs
    bionetAPI.timestepBionetworks()

      
class MitosisSteppable(MitosisSteppableBase):
  def __init__(self,_simulator,_frequency,_tV):
    MitosisSteppableBase.__init__(self,_simulator,_frequency)
    self.tV=_tV;

  def step(self,mcs):
    cells2divide=[]
    for cell in self.cellList:
      if cell:
        cellDict=CompuCell.getPyAttrib(cell)
        if (cellDict["CycB"]<=0.1 and cellDict["CycB0"]>0.1):
          cells2divide.append(cell)
    for cell in cells2divide:
      self.divideCellRandomOrientation(cell)

  def updateAttributes(self):  
    parentCell=self.mitosisSteppable.parentCell
    childCell=self.mitosisSteppable.childCell
     
    #transfering attributes
    childCell.type=parentCell.type
    bionetAPI.copyBionetworkFromParent(parentCell,childCell)
    #volume
    parentCell.targetVolume=parentCell.volume
    childCell.targetVolume=childCell.volume
    childCell.lambdaVolume=parentCell.lambdaVolume
    #dictionaries
    parentCellDict=CompuCell.getPyAttrib(parentCell)
    childCellDict=CompuCell.getPyAttrib(childCell)
    for key,item in parentCellDict.items():
      if (key!="Bionetwork"): 
        childCellDict[key] = deepcopy(parentCellDict[key])

    
class ExtraFields(SteppableBasePy):
  def __init__(self,_simulator,_frequency=1):
    SteppableBasePy.__init__(self,_simulator,_frequency)

  def setScalarFields(self,_field1,_field2):
    self.scalarField1=_field1
    self.scalarField2=_field2

  def step(self,mcs):
    clearScalarValueCellLevel(self.scalarField1)
    clearScalarValueCellLevel(self.scalarField2)
    for cell in self.cellList:
      if (cell):
        cellDict=CompuCell.getPyAttrib(cell)
        fillScalarValueCellLevel(self.scalarField1,cell,cellDict["CycB"])
        fillScalarValueCellLevel(self.scalarField2,cell,cellDict["Cdh1"])
