import sys,time
from os import environ
from os import getcwd
import string
sys.path.append(environ["PYTHON_MODULE_PATH"])
sys.path.append(environ["SWIG_LIB_INSTALL_DIR"])

global Lx; global Ly; global cd; global T; global nOrder
global R; global LamV; global LamS; global tV; global tS

#PARAMETERS:
cd=7            #cell diameter
Lx=10*cd        #Lattice size - x
Ly=10*cd        #Lattice size - y
R=4*cd          #Radius of initial distribution
T=10            #Temperature
nOrder=4        #Distance of interaction
#
#VOLUME/SURFACE PARAMETERS:
LamV=10        #Lambda Volume
tV=cd*cd       #Target Volume

def configureSimulation(sim):
   import CompuCellSetup
   from XMLUtils import ElementCC3D
   cc3d=ElementCC3D("CompuCell3D")
   potts=cc3d.ElementCC3D("Potts")
   potts.ElementCC3D("Dimensions",{"x":Lx,"y":Ly,"z":1})
   potts.ElementCC3D("Steps",{},10000)
   potts.ElementCC3D("Temperature",{},int(T))
   potts.ElementCC3D("NeighborOrder",{},2)
   potts.ElementCC3D("Boundary_x",{},"Periodic")
   potts.ElementCC3D("Boundary_y",{},"Periodic")
   
   cellType=cc3d.ElementCC3D("Plugin",{"Name":"CellType"})
   cellType.ElementCC3D("CellType", {"TypeName":"Medium","TypeId":"0"})
   cellType.ElementCC3D("CellType", {"TypeName":"TypeA", "TypeId":"1"})
   cellType.ElementCC3D("CellType", {"TypeName":"TypeB", "TypeId":"2"})

   #CONTACT
   contact=cc3d.ElementCC3D("Plugin",{"Name":"Contact"})
   contact.ElementCC3D("Energy", {"Type1":"Medium", "Type2": "Medium"},0)
   contact.ElementCC3D("Energy", {"Type1":"Medium", "Type2": "TypeA"},10)
   contact.ElementCC3D("Energy", {"Type1":"Medium", "Type2": "TypeB"},10)
   contact.ElementCC3D("Energy", {"Type1":"TypeA",  "Type2": "TypeA"},10)
   contact.ElementCC3D("Energy", {"Type1":"TypeA",  "Type2": "TypeB"},10)
   contact.ElementCC3D("Energy", {"Type1":"TypeB",  "Type2": "TypeB"},10)
   #-neighbor order
   contact.ElementCC3D("NeighborOrder",{},nOrder)
   
   #OTHER PLUGINS
   vlfpd = cc3d.ElementCC3D("Plugin",{"Name":"VolumeLocalFlex"})
   ntpd = cc3d.ElementCC3D("Plugin",{"Name":"NeighborTracker"})
   
   #CELLS INITIAL CONFIGURATION:
   blobInit=cc3d.ElementCC3D("Steppable",{"Type":"BlobInitializer"})
   blobInit.ElementCC3D("Width",{},cd)
   blobInit.ElementCC3D("CellSortInit",{},"yes")
   blobInit.ElementCC3D("Radius",{},R)
  
   #BOX WATCHER
   bwpd=cc3d.ElementCC3D("Steppable",{"Type":"BoxWatcher"})
   bwpd.ElementCC3D("XMargin",{}, 2)
   bwpd.ElementCC3D("YMargin",{}, 2)
   bwpd.ElementCC3D("ZMargin",{}, 1)
      
   CompuCellSetup.setSimulationXMLDescription(cc3d)

   
import CompuCellSetup
sim,simthread = CompuCellSetup.getCoreSimulationObjects()
configureSimulation(sim)

import CompuCell
CompuCellSetup.initializeSimulationObjects(sim,simthread)
pyAttributeAdder,dictAdder=CompuCellSetup.attachDictionaryToCells(sim)

#Add Python steppables here
steppableRegistry=CompuCellSetup.getSteppableRegistry()

from SimpleOscillator_Step import InitCond
initCond=InitCond(_simulator=sim,_frequency=1,_LamV=LamV,_tV=tV)
steppableRegistry.registerSteppable(initCond)

from SimpleOscillator_Step import Oscillator
oscillator=Oscillator(_simulator=sim,_frequency=1)
steppableRegistry.registerSteppable(oscillator)

#Create extra player fields here or add attributes
dim=sim.getPotts().getCellFieldG().getDim()
Field=simthread.createScalarFieldCellLevelPy("Osc")

from SimpleOscillator_Step import ExtraFields
extraFields=ExtraFields(_simulator=sim,_frequency=5)
extraFields.setScalarFields(Field)
steppableRegistry.registerSteppable(extraFields)

CompuCellSetup.mainLoop(sim,simthread,steppableRegistry)
##sys.exit()
