import os
import sys
import CompuCell
from PySteppables import *
from PlayerPython import *
from math import *


class InitCond(SteppableBasePy):
  def __init__(self,_simulator,_frequency,_LamV,_tV):
    SteppableBasePy.__init__(self,_simulator,_frequency)
    self.LamV=_LamV; self.tV=_tV
    
  def start(self):
    for cell in self.cellList:
      if (cell):
        cell.targetVolume=self.tV
        cell.lambdaVolume=self.LamV

        
import bionetAPI
class Oscillator(SteppableBasePy):
  def __init__(self,_simulator,_frequency):
    SteppableBasePy.__init__(self,_simulator,_frequency)
    bionetAPI.initializeBionetworkManager(self.simulator)
    
  def start(self):
    #Loading model
    ModelName = "SimpleOscillator"
    ModelKey  = "SO"
    ModelPath = os.getcwd() + "/Demos/DemosBionetSolver/SimpleOscillator/SimpleOscillator.sbml"
    print ModelPath
    IntegrationStep = 1.0
    bionetAPI.loadSBMLModel(ModelName, ModelPath, ModelKey, IntegrationStep)

    bionetAPI.addSBMLModelToTemplateLibrary(ModelName,"TypeA")
    bionetAPI.addSBMLModelToTemplateLibrary(ModelName,"TypeB")
    
    #Initial conditions
    bionetAPI.initializeBionetworks()
    bionetAPI.setBionetworkValue("SO_n",2,"TypeB")
      
  def step(self,mcs):
    for cell in self.cellList:
      if (cell):
        #Storing V1 and V2 levels of each cell
        cellDict=CompuCell.getPyAttrib(cell)
        cellDict["V1"]=bionetAPI.getBionetworkValue("SO_V1",cell.id)
        cellDict["V2"]=bionetAPI.getBionetworkValue("SO_V2",cell.id)
        #bionetAPI.setBionetworkValue("SO_n",random.random()*4,cell.id)
    #Iterating the ODEs
    bionetAPI.timestepBionetworks()


class ExtraFields(SteppableBasePy):
  def __init__(self,_simulator,_frequency=1):
    SteppableBasePy.__init__(self,_simulator,_frequency)

  def setScalarFields(self,_field1):
      self.scalarField1=_field1

  def step(self,mcs):
    clearScalarValueCellLevel(self.scalarField1)
    for cell in self.cellList:
      if (cell):
        cellDict=CompuCell.getPyAttrib(cell)
        fillScalarValueCellLevel(self.scalarField1,cell,cellDict["V1"])
