import sys,time
from os import environ
from os import getcwd
import string
sys.path.append(environ["PYTHON_MODULE_PATH"])
sys.path.append(environ["SWIG_LIB_INSTALL_DIR"])

#---------------------------------------------------------------- 
#  Toy model for SOMITE FORMATION
#     - The model has been simplified and optimized for fast 
#       simulation time and serves as a illustration of how to
#       build a complex multiscale model.
#     - For a more realistic somite simulation, please see:
#         Hester SD, Belmonte JB et al. PLoS Comp. Biol. 2011
#----------------------------------------------------------------

global LamV; global tV; global Grow
global Fgf0; global Kfgf; global f2w; global k1; global k2

#VOLUME/SURFACE PARAMETERS:
LamV=5          #Lambda Volume
tV=49           #Target Volume
Grow=4.00       #Tailbud cell's grow rate
#
#CLOCK-WAVEFRONT PARAMETERS:
Fgf0=45        #Initial concentration of Fgf8
Kfgf=0.0018    #Decay rate of Fgf8
f2w=1.6/Fgf0   #Conversion from Fgf8 to Wnt3a
k1=21.28       #ReadOut parameter
k2=0.400       #ReadOut parameter

def configureSimulation(sim):
   import CompuCellSetup
   from XMLUtils import ElementCC3D
   cc3d=ElementCC3D("CompuCell3D")
   potts=cc3d.ElementCC3D("Potts")
   potts.ElementCC3D("Dimensions",{"x":88,"y":462,"z":1})
   potts.ElementCC3D("Steps",{},2000)
   potts.ElementCC3D("Temperature",{},15)
   potts.ElementCC3D("NeighborOrder",{},2)
   
   #CELL TYPES:
   cellType=cc3d.ElementCC3D("Plugin",{"Name":"CellType"})
   cellType.ElementCC3D("CellType", {"TypeName":"Medium","TypeId":"0"})
   cellType.ElementCC3D("CellType", {"TypeName":"PSM",   "TypeId":"1"})
   cellType.ElementCC3D("CellType", {"TypeName":"Source","TypeId":"2"})
   cellType.ElementCC3D("CellType", {"TypeName":"Wall",  "TypeId":"3","Freeze":True})
   cellType.ElementCC3D("CellType", {"TypeName":"Core",  "TypeId":"4"})
   cellType.ElementCC3D("CellType", {"TypeName":"Eph",   "TypeId":"5"})
   cellType.ElementCC3D("CellType", {"TypeName":"ephrin","TypeId":"6"})

   #CONTACT ENERGIES:
   contact=cc3d.ElementCC3D("Plugin",{"Name":"Contact"})
   contact.ElementCC3D("Energy", {"Type1":"Medium", "Type2": "Medium"},0)
   contact.ElementCC3D("Energy", {"Type1":"Medium", "Type2": "PSM"},10)
   contact.ElementCC3D("Energy", {"Type1":"Medium", "Type2": "Source"},10)
   contact.ElementCC3D("Energy", {"Type1":"Medium", "Type2": "Wall"},0)
   #PSM
   contact.ElementCC3D("Energy", {"Type1":"PSM", "Type2": "PSM"},5)
   contact.ElementCC3D("Energy", {"Type1":"PSM", "Type2": "Source"},5)
   contact.ElementCC3D("Energy", {"Type1":"PSM", "Type2": "Wall"},10)
   #Source
   contact.ElementCC3D("Energy", {"Type1":"Source", "Type2": "Source"},5)
   contact.ElementCC3D("Energy", {"Type1":"Source", "Type2": "Wall"},15)
   #Wall
   contact.ElementCC3D("Energy", {"Type1":"Wall", "Type2": "Wall"},0)
   #Core
   contact.ElementCC3D("Energy", {"Type1":"Core", "Type2": "Medium"},10)
   contact.ElementCC3D("Energy", {"Type1":"Core", "Type2": "PSM"},10)
   contact.ElementCC3D("Energy", {"Type1":"Core", "Type2": "Source"},10)
   contact.ElementCC3D("Energy", {"Type1":"Core", "Type2": "Wall"},10)
   contact.ElementCC3D("Energy", {"Type1":"Core", "Type2": "Core"},4)
   contact.ElementCC3D("Energy", {"Type1":"Core", "Type2": "Eph"},8)
   contact.ElementCC3D("Energy", {"Type1":"Core", "Type2": "ephrin"},8)
   #Eph
   contact.ElementCC3D("Energy", {"Type1":"Eph", "Type2": "Medium"},10)
   contact.ElementCC3D("Energy", {"Type1":"Eph", "Type2": "PSM"},10)
   contact.ElementCC3D("Energy", {"Type1":"Eph", "Type2": "Source"},10)
   contact.ElementCC3D("Energy", {"Type1":"Eph", "Type2": "Wall"},10)
   contact.ElementCC3D("Energy", {"Type1":"Eph", "Type2": "Eph"},8)
   contact.ElementCC3D("Energy", {"Type1":"Eph", "Type2": "ephrin"},30)
   #ephrin
   contact.ElementCC3D("Energy", {"Type1":"ephrin", "Type2": "Medium"},10)
   contact.ElementCC3D("Energy", {"Type1":"ephrin", "Type2": "PSM"},10)
   contact.ElementCC3D("Energy", {"Type1":"ephrin", "Type2": "Source"},10)
   contact.ElementCC3D("Energy", {"Type1":"ephrin", "Type2": "Wall"},10)
   contact.ElementCC3D("Energy", {"Type1":"ephrin", "Type2": "ephrin"},8)
   #-neighbor order
   contact.ElementCC3D("NeighborOrder",{},4)
   
   #OTHER PLUGINS
   ptp  = cc3d.ElementCC3D("Plugin",{"Name":"PixelTracker"})
   vlfp = cc3d.ElementCC3D("Plugin",{"Name":"VolumeLocalFlex"})
   ntp  = cc3d.ElementCC3D("Plugin",{"Name":"NeighborTracker"})
   comp = cc3d.ElementCC3D("Plugin",{"Name":"CenterOfMass"})
   
   #CELLS INITIAL CONFIGURATION:
   pifi = cc3d.ElementCC3D("Steppable",{"Type":"PIFInitializer"})
   pifi.ElementCC3D("PIFName",{},os.getcwd()+"\DemosBionetSolver\Somites\InitialCondition.piff")
   
   #BOX WATCHER
   bwpd=cc3d.ElementCC3D("Steppable",{"Type":"BoxWatcher"})
   bwpd.ElementCC3D("XMargin",{},2)
   bwpd.ElementCC3D("YMargin",{},2)
   bwpd.ElementCC3D("ZMargin",{},1)
      
   CompuCellSetup.setSimulationXMLDescription(cc3d)

import CompuCellSetup
sim,simthread = CompuCellSetup.getCoreSimulationObjects()
configureSimulation(sim)

import CompuCell
# Create extra player fields here or add attributes
pyAttributeAdder,dictAdder=CompuCellSetup.attachDictionaryToCells(sim)
CompuCellSetup.initializeSimulationObjects(sim,simthread)

#Add Python steppables here
steppableRegistry=CompuCellSetup.getSteppableRegistry()

from Somites_Step import InitCond
initCond=InitCond(_simulator=sim,_frequency=1,_LamV=LamV,_tV=tV)
steppableRegistry.registerSteppable(initCond)

from Somites_Step import Gradients
gradients=Gradients(_simulator=sim,_frequency=1,_Fgf0=Fgf0,_Kfgf=Kfgf,_f2w=f2w)
steppableRegistry.registerSteppable(gradients)

from Somites_Step import Clock
clock=Clock(_simulator=sim,_frequency=1)
steppableRegistry.registerSteppable(clock)

from Somites_Step import Tailbud
tailbud=Tailbud(_simulator=sim,_frequency=1,_tV=tV,_Grow=Grow)
steppableRegistry.registerSteppable(tailbud)

from Somites_Step import ReadOut
readOut=ReadOut(_simulator=sim,_frequency=1,_k1=k1,_k2=k2)
steppableRegistry.registerSteppable(readOut)

#Create extra player fields here or add attributes
dim=sim.getPotts().getCellFieldG().getDim()
Fgf8Field=simthread.createScalarFieldCellLevelPy("Fgf8")
LfngField=simthread.createScalarFieldCellLevelPy("Lfng")

from Somites_Step import ExtraFields
extraFields=ExtraFields(_simulator=sim,_frequency=5)
extraFields.setScalarFields(Fgf8Field,LfngField)
steppableRegistry.registerSteppable(extraFields)

CompuCellSetup.mainLoop(sim,simthread,steppableRegistry)
##sys.exit()