from PySteppables import *
import CompuCell
import sys
from PlayerPython import *
from math import *
from PySteppablesExamples import MitosisSteppableBase
import os
from copy import deepcopy

class InitCond(SteppableBasePy):
  def __init__(self,_simulator,_frequency,_LamV,_tV):
    SteppableBasePy.__init__(self,_simulator,_frequency)
    self.LamV=_LamV; self.tV=_tV
    
  def start(self):
    for cell in self.cellList:
      if (cell):
        cell.targetVolume=self.tV
        cell.lambdaVolume=self.LamV

        
class Gradients(SteppableBasePy):
  def __init__(self,_simulator,_frequency,_Fgf0,_Kfgf,_f2w):
    SteppableBasePy.__init__(self,_simulator,_frequency)
    self.Fgf0=_Fgf0; self.Kfgf=_Kfgf; self.f2w=_f2w
    
  def start(self): #setting inital gradient distribution
    for cell in self.cellList:
      if (cell and cell.type!=3):
        cellDict=CompuCell.getPyAttrib(cell)
        if (cell.type==2):
          cellDict["Fgf8"]=self.Fgf0
        elif (cell.type==1):
          cellDict["Fgf8"]=self.Fgf0*(1-self.Kfgf)**(20)
        cellDict["Wnt3a"]=self.f2w*cellDict["Fgf8"]

  def step(self,mcs): #Calcutating gradients
    for cell in self.cellList:
      if (cell and cell.type!=3):
        cellDict=CompuCell.getPyAttrib(cell)
        if (cell.type==1):  
          cellDict["Fgf8"]=cellDict["Fgf8"]*(1-self.Kfgf)
        elif (cell.type==2):
          cellDict["Fgf8"]=self.Fgf0
        cellDict["Wnt3a"]=self.f2w*cellDict["Fgf8"]

        
import bionetAPI        
class Clock(SteppableBasePy):
  def __init__(self,_simulator,_frequency):
    SteppableBasePy.__init__(self,_simulator,_frequency)
    bionetAPI.initializeBionetworkManager(self.simulator)
    
  def start(self):
    #Loading model
    ModelName = "HesterBelmonte2011"
    ModelKey = "HB"
    ModelPath = os.getcwd()+"/DemosBionetSolver/Somites/HesterBelmonte2011.sbml"
    IntegrationStep = 0.4
    bionetAPI.loadSBMLModel(ModelName, ModelPath, ModelKey, IntegrationStep)
    
    bionetAPI.addSBMLModelToTemplateLibrary(ModelName,"PSM")
    bionetAPI.addSBMLModelToTemplateLibrary(ModelName,"Source")
    bionetAPI.initializeBionetworks()
    
    #Initial conditions
    for cell in self.cellList:
      if (cell and cell.type<3):
        cellDict=CompuCell.getPyAttrib(cell)
        cellDict["Lfng"]=bionetAPI.getBionetworkValue("HB_F",cell.id)
        cellDict["Axin"]=bionetAPI.getBionetworkValue("HB_A",cell.id)
        cellDict["Beta"]=bionetAPI.getBionetworkValue("HB_B",cell.id)
        cellDict["DLm"]=bionetAPI.getBionetworkValue("HB_DLm",cell.id)
        cellDict["extD"]=bionetAPI.getBionetworkValue("HB_DLm",cell.id)  
      
  def step(self,mcs):
    for cell in self.cellList:
      if (cell and cell.type<3):
        cellDict=CompuCell.getPyAttrib(cell)
        cellDict["Lfng"]=bionetAPI.getBionetworkValue("HB_F",cell.id)
        cellDict["Axin"]=bionetAPI.getBionetworkValue("HB_A",cell.id)
        cellDict["Beta"]=bionetAPI.getBionetworkValue("HB_B",cell.id)
        cellDict["extD"]=0; extD=0.0; n=0
        cellNeighborList=self.getCellNeighbors(cell)
        for neighbor in cellNeighborList:
          if (neighbor.neighborAddress and neighbor.neighborAddress.type<3):
            neighDict=CompuCell.getPyAttrib(neighbor.neighborAddress)
            extD+=neighDict["DLm"]; n+=1
        if (n>0):
          cellDict["extD"]=extD/n
    for cell in self.cellList:
      if (cell and cell.type<3):  
        cellDict=CompuCell.getPyAttrib(cell)
        bionetAPI.setBionetworkValue("HB_DLm_ext",cellDict["extD"],cell.id)
        bionetAPI.setBionetworkValue("HB_Fgf",cellDict["Fgf8"],cell.id)
        bionetAPI.setBionetworkValue("HB_D",cellDict["Wnt3a"],cell.id)
        cellDict["DLm"]=bionetAPI.getBionetworkValue("HB_DLm",cell.id)
    #Iterating the ODEs
    bionetAPI.timestepBionetworks()

    
class Tailbud(MitosisSteppableBase):
  def __init__(self,_simulator,_frequency,_tV,_Grow):
    MitosisSteppableBase.__init__(self,_simulator,_frequency)
    self.tV=_tV; self.Grow=_Grow
  
  def step(self,mcs):
    #CHECKING CELL TYPES:
    for cell in self.cellList:
      if (cell):   
        if (cell.type==1): #PSM
          cellDict=CompuCell.getPyAttrib(cell)
          if (cellDict["Fgf8"]>25):
            cellNeighborList=self.getCellNeighbors(cell)
            for neighbor in cellNeighborList:
              if not neighbor.neighborAddress: #PSM cells --> Source
                cell.type=2 
        elif (cell.type==2): #Source
          med=0; cellNeighborList=self.getCellNeighbors(cell)
          for neighbor in cellNeighborList:
            if not neighbor.neighborAddress: 
              med=1
          if (med==0): #Source cells --> PSM cells
              cell.type=1; cell.targetVolume=self.tV
          else: #turning grow off when the bottom is reached
            if (cell.yCOM<30):
              cell.type=1; cell.targetVolume=self.tV
    #GROWTH AND DIVISION:
    cells2divide=[]
    for cell in self.cellList:
      if (cell and cell.type==2):
        cell.targetVolume+=self.Grow
        if (cell.volume>=2*self.tV):
          cells2divide.append(cell)
    for cell in cells2divide:
      self.divideCellOrientationVectorBased(cell,0,1,0)
          
  def updateAttributes(self):
    parentCell=self.mitosisSteppable.parentCell
    childCell=self.mitosisSteppable.childCell
    childCell.type=2
    #clock
    bionetAPI.copyBionetworkFromParent(parentCell,childCell)
    #volume
    parentCell.targetVolume=self.tV
    childCell.targetVolume=parentCell.targetVolume
    childCell.lambdaVolume=parentCell.lambdaVolume
    #dictionaries
    parentCellDict=CompuCell.getPyAttrib(parentCell)
    childCellDict=CompuCell.getPyAttrib(childCell)
    for key,item in parentCellDict.items():
      if (key!="Bionetwork"): 
        childCellDict[key] = deepcopy(parentCellDict[key])

        
class ReadOut(SteppableBasePy):
  def __init__(self,_simulator,_frequency,_k1,_k2):
    SteppableBasePy.__init__(self,_simulator,_frequency)
    self.k1=_k1; self.k2=_k2; self.wallOff=self.dim.y

  def step(self,mcs):
    for cell in self.cellList:
      if (cell and cell.type==1): #Clock Read-Out
        cellDict=CompuCell.getPyAttrib(cell)
        if (cellDict["Fgf8"]<14):
          if (cellDict["Beta"]>self.k2):
            cell.type=4 #Highg N-Cadherin 
            print "EPHA4"
          elif (cellDict["Lfng"]/cellDict["Axin"]>self.k1):
            cell.type=5 #High EphA4
          else:
            cell.type=6 #High ephrinB2    
          self.wallOff=min(self.wallOff,cell.yCOM)
      elif (cell.type==3): #Erasing walls
        pt=CompuCell.Point3D(); pt.x=0; pt.y=0; pt.z=0
        mediumCell=self.cellField.get(pt)
        pixelList=CellPixelList(self.pixelTrackerPlugin,cell)
        for pixelData in pixelList:
          pt=pixelData.pixel
          if (pt.y>=(self.wallOff-7)):
            self.cellField.set(pt,mediumCell)
 
 
class ExtraFields(SteppableBasePy):
  def __init__(self,_simulator,_frequency=1):
    SteppableBasePy.__init__(self,_simulator,_frequency)

  def setScalarFields(self,_field1,_field2):
    self.scalarField1=_field1
    self.scalarField2=_field2
  def step(self,mcs):
    clearScalarValueCellLevel(self.scalarField1)
    clearScalarValueCellLevel(self.scalarField2)
    for cell in self.cellList:
      if (cell and cell.type<3):
        cellDict=CompuCell.getPyAttrib(cell)
        fillScalarValueCellLevel(self.scalarField1,cell,cellDict["Fgf8"])
        fillScalarValueCellLevel(self.scalarField2,cell,cellDict["Lfng"])
      elif (cell.type==3):
        fillScalarValueCellLevel(self.scalarField1,cell,45)
        fillScalarValueCellLevel(self.scalarField2,cell,1.6)